#############################################################
# R script illustrating the analysis of undirected networks #
#############################################################

# load RSiena commands:
library(RSiena)

# set working directory to where the data are:
setwd("<put path to your data directory here>")
list.files()

# Read data sets:
net1 <- as.matrix(read.table("KAPFTS1.dat"))
net2 <- as.matrix(read.table("KAPFTS2.dat"))
job.title <- as.matrix(read.table("kapfa_stat.dat")) 
attribs <- as.matrix(read.table("kapfat.dat")) 
# for contents and coding, see Siena webpage / file "kapfdata.htm"

# No need to recode data or identify missings (this data set does not
# contain any missing data).
nractors <- length(job.title)
nractors # there are 39 of them

# Identify dependent network variable:
socialising <- sienaNet(array(c(net1,net2),dim=c(nractors,nractors,2)))

# Identify predictor variables:
jobtitle <- coCovar(job.title[,1])
jobstatus <- coCovar(attribs[,2])
Lyashi <- coCovar(attribs[,1])

# Bind data together for Siena analysis:
undirectedData <- sienaDataCreate(socialising,jobtitle,jobstatus,Lyashi)

# Create effects object for model specification:
undirectedEffects <- getEffects(undirectedData)

# Write first descriptive results to protocol file (optional):
print01Report(undirectedData,undirectedEffects,modelname = 'undirected-init')

# Before specifying a model, take a look at what is possible:
effectsDocumentation()
# check out the parts titled 'symmetricObjective' and 'covarSymmetricObjective'
# in particular. Note that some effect names are different (transTriads), while
# others are not available because meaningless in symmetrical networks, such as
# 'reciprocity' or 'cycle3'.

# Now specify the model:
undirectedEffects <- includeEffects(undirectedEffects,transTriads)
undirectedEffects <- includeEffects(undirectedEffects,altX,interaction1="Lyashi")
undirectedEffects <- includeEffects(undirectedEffects,altX,interaction1="jobstatus")
undirectedEffects <- includeEffects(undirectedEffects,sameX,interaction1="jobtitle")
undirectedEffects <- includeEffects(undirectedEffects,simX,interaction1="jobstatus")
undirectedEffects
# The main effect of 'Lyashi' was included because Kapferer identified him as a
# particularly network-active person, a potential statistical outlier. We will
# investigate the degree to which his special treatment affected results in the end.

# Before creating the model object, take a look at what is special:
?sienaModelCreate
# Take a close look at the description of the "modelType" option.

# Now finally create the model object; let's choose modelType=3 (which stands 
# shorthand for "unilateral initiative and reciprocal confirmation required for a 
# tie to be formed; unilateral action suffices for a tie to be terminated"):
undirectedModel <- sienaModelCreate(useStdInits=TRUE,modelType=3,projname='undirected-results')

# Estimate the model:
undirectedResults <- siena07(undirectedModel,data=undirectedData,
  effects=undirectedEffects,batch=FALSE,verbose=FALSE)
undirectedResults
# What we see is evidence for transitive closure, homophily on jobtitle, and
# an effect of Lyashi indicating his higher involvement in the network.
# There are no (or at best weak) effects of job status on sich involvement
# (t-ratio about 1.5) or similarity in terms of job status (t-ratio about 1.7).

# Now let's see what will happen when we cease to control for the 'Lyashi'
# outlier indicator:
undirectedEffects$include[undirectedEffects$shortName=="altX" &
  undirectedEffects$interaction1=="Lyashi"] <- FALSE
# NOTE that we do NOT use the 'setEffect' function here because there
# seems to be a bug in the software for this effect & symmetric networks
# (as of July 2011).  
undirectedEffects

# Obtain new results for the reduced model:
newundirectedResults <- siena07(undirectedModel,data=undirectedData,
  effects=undirectedEffects,batch=FALSE,verbose=FALSE)
# results in the 'output file' will be appended in the end.
newundirectedResults
# The only difference seems to be that now 'jobstatus' is borderline 
# significant, which can be explained by Lyashi's high job status. When he
# is not distinguished from others, his whole status group is identified as
# more involved in the network.

# So much for the undirected options in this script. You can try estimating
# the same models with other 'modelType' choices, if you like. Note that
# tha rate function means something qualitatively different for modelType
# values 4,5, and 6 (pair-based decisions to swap a tie) than for values
# 2 and 3 (actor-based decisions to choose one partner as in the 'directed'
# case).
